<?php
/*----------------------------------------------------------------------------------------------
| Kiss Upload Class								Version	1.5.59
|-----------------------------------------------------------------------------------------------
| Author: Heiner Klostermann (DocChicago) - KISS Software Germany				
|		  Andrey Yasinecky <andrey@yasinecky.com>
| Copyright (c) 2010 by KISS Software, 59227 Ahlen (NRW), Germany				
|-----------------------------------------------------------------------------------------------
| License GNU/GPL 					http://www.gnu.org/copyleft/gpl.html
|-----------------------------------------------------------------------------------------------
|												
| This component is free software, distributed under the GNU / GPL License. You may use,	
| copy and redistribute it free of charge as long as it complies with the license terms.	
|												
| 
| For help and suggestions please visit our support forum at www.kiss-software.de
|
|-----------------------------------------------------------------------------------------------
| created 2010-01-01							updated	2010-11-22
|-----------------------------------------------------------------------------------------------*/

class Upload
{

	/**
     * Max file size
     * @var  	 integer
     * @access   private
     */
	var $max_file_size = 1572864; //1,5 Mb
	
	/**
     * Max image width
     * @var  	 integer
     * @access   private
     */
	var $max_image_width = 1024; //pixels

	/**
     * Upload Rights
     * @var  	 integer
     * @access   private
     */
	var $upload_rights = 0755; //file rights
	
	/**
	 * an array which contains uploaded files
	 * @var array
	 * @access private
	 */
	var $_files = array();

	/**
	 * path to files's icons folder
	 * @var string
	 * @access private
	 */
	var $icons_path = 'ico/';

	/**
	 * array which contains valid filetypes
	 * @var array
	 * @access private
	 */
	var $valid_types = array('jpg', 'jpeg', 'png', 'doc', 'txt', 'gif', 'xml', 'ldif', 'ldf', 'mdb', 'csv', 'dbf', 'vcf', 'rtf', 'pdf', 'xls', 'rar', 'tar', 'zip', 'tgz', 'gz');

	/**
	 * Error Message
	 * @var string
	 * @access private
	 */
	var $error = '';

    /**
     * Class constructor
     * @param    integer    $file_size    max file size
     * @param    string     $target_dir   dir to upload
     * @access   public
     */

	
	// }}}
    // {{{ setMaxWidth()

    /**
     * Setting max image width
     */
    function setMaxWidth($width)
    {
    	$this->max_image_width = $width;
    }
    
	// }}}
    // {{{ setMaxWidth()

    /**
     * Setting Upload Rights
     */
    function setRights($rights)
    {
//    	$this->upload_rights = 0755;
    	$this->upload_rights = $rights;
    }

    // }}}
    // {{{ setMaxSize()

    /**
     * Setting max file size
     */
    function setMaxSize($size)
    {
    	$this->max_file_size = $size;
    }

    /**
     * Setting valid filetypes
     */
    function setValidTypes($filetypes)
    {
	$valtypes = explode(',', str_replace(' ', '', $filetypes));
    	$this->valid_types = $valtypes;
    }

    // }}}
    // {{{ checkDir()

    /**
     * Check destination dir. If dir not exists, create new one.
     *
     * @param string  $dir  destination folder
     * @return absolute path to dir
     * @access  public
     */
	function checkDir($dir)
	{
	    $dir = preg_replace("/(.*)(\/)$/","\\1", $dir);

		$absolute_dir = $dir;
		if (!is_dir($absolute_dir)) {
			@mkdir($absolute_dir, $this->upload_rights);
			// eval("chmod(\"".$absolute_dir."\", 0755);"); // if php script has special rights to execute chmod
			// @chmod($absolute_dir, $this->upload_rights);
			$filemode = "0644";
			$foldermode = $this->upload_rights;
			if (JPath::canChmod($absolute_dir)) {
				JPath::setPermissions($absolute_dir, $filemode, $foldermode);					
				}
		}
		return $absolute_dir;
	} // end checkDir()


	// }}}
    // {{{ checkGd()

    /**
     * Check GD library version.
     *
     * @return  GD version or false
     * @access  public
     */
	function checkGd()
	{
		$gd_content = get_extension_funcs('gd'); // Grab function list
		if (!$gd_content) { 
			$this->message_err('Upload class: GD libarary is not installed!', '', '', E_USER_ERROR); 
			return false; 
		} else {
			ob_start();
			phpinfo(8);
			$buffer = ob_get_contents();
			ob_end_clean(); 

			if (strpos($buffer, '2.0')) {
			    return 'gd2';
			} else {
			    return 'gd';
			}
		}
	}	
	

	// }}}
    // {{{ deleteFile()

    /**
     * Delete file from server
     *
     * @param string  $filename  filename
     * @param string  $source_dir   the dir in which is file
     * @return bool true|falses
     * @access  public
     */
	function deleteFile($filename, $source_dir)
	{
	    $source_dir = $this->checkDir($source_dir);
		if (file_exists($source_dir.DS.$filename)) {
			// Try to chmod file rights for delete
			@chmod($source_dir.DS.$filename, 0777);
			if (unlink($source_dir.DS.$filename)) {
				return true;
			}
		}
		return false;
	}


	// }}}
    // {{{ latinEncode()

    /**
     * Encode string from russian cyrillic to latin
     *
     * @param string  $str
     * @return string $new_string
     * @access  public
     */
	function latinEncode($str)
	{
	    $accordance = array('a','b','v','g','d','e','zh','z','i','y','k','l','m','n','o','p',
							'r','s','t','u','f','h','ch','c','sh','sh','','i','','e','u','ya');
		$cirilyc = array('a','�','�','�','�','�','�','�','�','�','�','�','�','�','�','�',
						 '�','�','�','�','�','�','�','�','�','�','�','�','�','�','�','�');
		$new_str = '';
		for ($i=0; $i<strlen($str); $i++) {
		    $symbol = substr($str, $i, 1);
			for($j=0; $j<count($cirilyc); $j++) {
			    if($symbol == $cirilyc[$j]) {
				    $symbol = $accordance[$j];
				}
			}
			$new_str.= $symbol;
		}
		
		return $new_str;
	}


    // }}}
	// {{{ uploadFile()
	
	/**
	 * Upload file to server
	 * 
	 * @param string $target_dir destination dir
	 * @param string $encode encode type
	 * @param string $name_length filename length
	 * @access public
	 */ 	
	function uploadFile($target_dir = '', $encode = 'md5', $name_length = 10, $resize = 0)
	{
	    $target_dir = $this->checkDir($target_dir);
	    $i=0;
	    foreach ($_FILES['name']['tmp_name'] as $key => $value) {
		    if (!empty($_FILES['name']['name'][$i])) {
			    if (is_uploaded_file($_FILES['name']['tmp_name'][$i])) {
				    $filename = strtolower(str_replace(' ', '', $_FILES['name']['name'][$i]));
					$basefilename = preg_replace("/(.*)\.([^.]+)$/","\\1", $filename);
					$ext = preg_replace("/.*\.([^.]+)$/","\\1", $filename);		
					if ($_FILES['name']['size'][$i] > $this->max_file_size) {
					    $this->message_err('ERR IMPOSSIBLE FILE SIZE', '', '', E_USER_WARNING);
					    return false;
					} elseif (!in_array($ext, $this->valid_types)) {
					    $this->message_err('ERR IMPOSSIBLE FILE TYPE', '', '', E_USER_WARNING);
					    return false;						
					} else {
					    // encode filename
					    switch ($encode) {
						case 'md5':
						    $basefilename = substr(md5($basefilename), 0, $name_length);
							$filename = $basefilename.'.'.$ext;
						    break;
							
						case 'latin':
						    $basefilename = substr($this->latinEncode($basefilename), 0, $name_length);
							$filename = $basefilename.'.'.$ext;
						    break;

						case 'original':
						    $basefilename = substr($basefilename, 0, $name_length);
							$filename = $basefilename.'.'.$ext;
						    break;
						}
						if (!move_uploaded_file($_FILES['name']['tmp_name'][$i], $target_dir.DS.$filename)) {
						    $this->message_err('ERR CANNOT UPLOAD FILE', '', '', E_USER_ERROR);
							return false;							
						} else {
						    chmod($target_dir.DS.$filename, $this->upload_rights);
							$filemode = $this->upload_rights;
							if (JPath::canChmod($target_dir)) {
								JPath::setPermissions($target_dir, $filemode);					
								}
							
						}
						
						//$this->_files[$varname] = $filename;
					}
				} else {
				    $this->message_err('ERR CANNOT CREATE TEMP FILE', '', '', E_USER_ERROR);
					return false;					
				}
			} else {
	    		if ($i == 0) {
			    $this->message_err('ERR FILES UPLOAD ARRAY EMPTY', '', '', E_USER_ERROR);
			    return false;
			    }
			}
		//File is uploaded, now check whether it must be resized
		if ($resize > 0 ) {
		    if (!strpos('jpg,jpeg,png', $ext) === false) {
			$this->imgResize($filename, $target_dir, $resize, false);
			}
		   }
		  $i++;
		}
	}
	
	
	// }}}
	// {{{ imgResize()
	
	/**
	 * Resize image on the server
	 * 
	 * @param string $filename the filename of image
	 * @param string $source_dir the dir where this image stored
	 * @param string $dest_width destination image width
	 * @param bool $duplicate set true if you want to duplicate image 
	 * @return bool true|false
	 * @access public
	 */ 	
	function imgResize($filename, $source_dir, $dest_width, $duplicate = false)
	{
	    $source_dir = $this->checkDir($source_dir);
		$full_path = $source_dir.'/'.$filename;
		$basefilename = preg_replace("/(.*)\.([^.]+)$/","\\1", $filename);
		$ext = preg_replace("/.*\.([^.]+)$/","\\1", $filename);

		switch ($ext) {
		case 'png':
		    $image = imagecreatefrompng($full_path);
		    break;
		
		case 'jpg':
		    $image = imagecreatefromjpeg($full_path);
		    break;
		
		case 'jpeg':
		    $image = imagecreatefromjpeg($full_path);
		    break;
		
		default:
		    $this->message_err('ERR WRONG GDVERSION FORMAT', '', '', E_USER_ERROR);
		    return false;
		    break;
		}

		$image_width = imagesx($image);
		$image_height = imagesy($image);
		// Do not enlarge images if smaller than maxwidth
		if ($image_width < $this->max_image_width) {
			exit();
			}
		// resize image pro rata
		if ($dest_width > $this->max_image_width) {$dest_width == $this->max_image_width;}
		$coefficient = ($image_width > $dest_width) ? (real)($dest_width / $image_width) : 1;
		$dest_width = (int)($image_width * $coefficient);
		$dest_height = (int)($image_height * $coefficient);

		// create copy of original image and work with that copy.
		// keeps the original image untouched
		if (false !== $duplicate) {
		    $filename = $basefilename.'_'.$dest_width.'.'.$ext;
		    copy($full_path, $source_dir.DS.$filename);
		}
		
		if ('gd2' == $this->checkGd()) { 
		    $img_id = imagecreatetruecolor($dest_width, $dest_height);
    		imagecopyresampled($img_id, $image, 0, 0, 0, 0, $dest_width + 1, $dest_height + 1, $image_width, $image_height);
		} else {
		    $img_id = imagecreate($dest_width, $dest_height);
    		imagecopyresized($img_id, $image, 0, 0, 0, 0, $dest_width + 1, $dest_height + 1, $image_width, $image_height);
		}

        switch ($ext) {
		case 'png':
		    imagepng($img_id, $source_dir.DS.$filename);
		    break;
		
		case 'jpg':
		    imagejpeg($img_id, $source_dir.DS.$filename);
		    break;
		
		case 'jpeg':
		    imagejpeg($img_id, $source_dir.DS.$filename);
		    break;
        }
		
		imagedestroy($img_id);
		
		return true;
	}

	
	// }}}
    // {{{ displayFiles()

    /**
     * Display files in destination folder 
     * @param $source_dir the dir where files stored
     * @access  public
     */
	function displayFiles($source_dir)
	{
	    $source_dir = $this->checkDir($source_dir);
		if ($contents = opendir($source_dir)) {
		    echo 'Directory contents:<br>';
			
			while (false !== ($file = readdir($contents))) {
			    if (is_dir($file)) continue;

			    $filesize = (real)(filesize($source_dir.DS.$file) / 1024);
				$filesize = number_format($filesize, 3, ',', ' ');

			    $ext = preg_replace("/.*\.([^.]+)$/","\\1", $file);
			    echo '<p><img src="'.$this->icons_path.''.$ext.'.gif">&nbsp;'.$file.'&nbsp;('.$filesize.') Kb</p>';
			}
		}
	}

	
	// }}}
    // {{{ fileRename()

    /**
     * Rename target file
     * @param $source_dir the dir where files stored
	 * @param $filename the source filename
	 * @param $newname new filename
     * @access  public
     */
	function fileRename($source_dir, $filename, $newname)
	{
	    $source_dir = $this->checkDir($source_dir);
	    chmod($source_dir.DS.$filename, 0777);		
		if (rename($source_dir.DS.$filename, $newname)) {
			chmod($source_dir.DS.$newname, $this->upload_rights);				
		    return true;
			} else {
		    return false;
		}
	}

	// }}}
    // {{{ message_err()

    /**
     * Trigger error
	 */
	function message_err($error_msg, $err_line, $err_file, $error_type = E_USER_WARNING)
    {
		//trigger_error($error_msg, $error_type);
		$err = JText::_($error_msg);	
		$err.= (!empty($err_line)) ? ' Line '.$err_line : '';
		$err.= (!empty($err_file)) ? ' in - '.$err_file : '';
		$this->error = $err;
    }
}
?>